function [x,y,s]=fiipm(At,b,c,K,options)
% adaptive update Full NT-step Infeasible IPM for symmetric optimization
% [x,y,s]=fiipm(At,b,c,K,options)
% Input Parameters:
%   At:  n \times m coefficient matrix
%   b:  R^{m}
%   c:  R^{n}
%   K: 
%       K.l:   number of NONNEGATIVE components
%       K.q:   lists the dimensions of LORENTZ (quadratic, second-order cone) constraints
%       K.s:   lists the dimensions of POSITIVE SEMI-DEFINITE (PSD)
%   options (optional):
%      options.epsilon (optional): absolute accuracy parameter 
%      options.zeta (optional): starting parameter
%      options.fid (optional): output device

% symmetric optimization problem:
% Let $\mathcal{J}$ be a Euclidean Jordan algebra with dimension $n$, rank $r$,
% and cone of squares (or its associated symmetric cone) $\mathcal{K}$.
% Consider the primal-dual pair of symmetric optimization problems
% \begin{equation}
% \tag{CP}
% \min\left\{\langle c, x \rangle : Ax=b,\ x\in\mathcal{K} \right\}
% \end{equation}
% and
% \begin{equation}
% \tag{CD}
% \max\left\{b^{T}y : A^{T}y+s=c,\ y\in\mathbf{R}^{m},\ s\in\mathcal{K} \right\}.
% \end{equation}
% Here $c$ and the rows of $A$ lies in $\mathcal{J}$, and $b \in \mathbf{R}^{m}$.
% Moreover, assume that $a_{i}$ is the $i$-th row of $A$,
% then $Ax=b$ means that
% \begin{equation*}
% \langle a_{i}, x\rangle=b_{i}, \quad\text{for each }i=1, \ldots, m,
% \end{equation*}
% while $A^{T}y+s=c$ means
% \begin{equation*}
% \sum_{i=1}^{m}y_{i}a_{i}+s=c.
% \end{equation*}

% turn off the warnings
warning off all;
% check the input parameters
if nargin<4
    fprintf('Not enough parameters to call soass.m\n');
    x=[]; 
    y=[];
    s=[];
    return;
end
if size(At,2)~=length(b)
    At=At';
end
% initialize K
if isfield(K,'l')
    if isempty(K.l)==1
        K.l=0;
    end
else
    K.l=0;
end
K.lq=K.l;
if isfield(K,'q')
    for i=1:1:length(K.q)
        K.lq=K.lq+K.q(i);
    end
end
% reorder the columns of At and b accordingly for handling sparsity of SDP
if isfield(K,'s') && issparse(At)
    [At,b,K,ord]=order(At,b,K);
end
% calculate the rank of the corresponding Euclidean Jordan algebra
K.r=sc_rk(K);
% handle the options
zeta=[];
epsilon=[];
fid=1;
if nargin>4
    % handle zeta
    if isfield(options,'zeta')
        if isempty(options.zeta)==0
            zeta=options.zeta;
        end
    end
    % handle epsilon
    if isfield(options,'epsilon')
        if isempty(options.epsilon)==0
            epsilon=options.epsilon;
        end
    end
    % handle fid
    if isfield(options,'fid')
        if isempty(options.fid)==0
           fid=options.fid;
        end
    end
end
if isempty(zeta)
    zeta=estimatezeta(At,b,c,K);
end
if isempty(epsilon)
    epsilon=estimateepsilon(zeta,K.r);
end
fprintf(fid,'zeta=%e    epsilon=%3.1e\n',zeta,epsilon);
% parameters of the algorithm
% barrier update parameter theta
theta0=1/(4*K.r);
% threshold parameter tau
tau=1/16;
% construct the starting point
x=zeta*identity(length(c),K);
y=zeros(length(b),1);
s=x;
nu=1;
mu=zeta*zeta;
% primal and dual residuals
rb=b-Ax(At',x,K);
rc=c-At*y-s;
dualitygap=sc_ip(x,s,K);
normrb=norm(rb);
normrc=sqrt(sc_ip(rc,rc,K));
% iterations of the algorithm
fprintf(fid,'it   delta1     theta     c_it delta2     nu              mu              duality gap     norm rb         norm rc\n');
iter=0;
while max([dualitygap,normrb,normrc])>epsilon
    iter=iter+1;
    % feasibility step
    [Dxa,Dya,Dsa,Dxc,Dyc,Dsc,theta]=NTf(At,K,mu,rb,rc,x,s);
    if theta<theta0
        fprintf(fid,'Terminate: there is no optimal solution satifies the starting point condition.\n');
        fprintf(fid,'primal object value=%e\n',sc_ip(c,x,K));
        fprintf(fid,'dual object value=%e\n',b'*y);
        return;
    end
%     if nu>1.1e-16 && (1-theta)*nu<1e-16
%         theta=1-1e-16/nu;
%         fprintf(fid, '*');
%     end
    % update parameters
    nu=(1-theta)*nu;
    mu=(1-theta)*mu;
    % update the iterates
    x=x+Dxc+theta*Dxa;
    y=y+Dyc+theta*Dya;
    s=s+Dsc+theta*Dsa;
    % centering steps
    d=delta(x,s,mu,K);
    fprintf(fid,'%2d   %f   %f',iter,d,theta);
%     if d>1/sqrt(2)
%         fprintf(fid,'Nummerical problems! Unexpected: delta>1/sqrt(2) after the feasibility step.\n');
%         fprintf(fid,'primal object value=%e\n',sc_ip(c,x,K));
%         fprintf(fid,'dual object value=%e\n',b'*y);
%         return;
%     end
    initer=0;
%     while d>tau||initer==0
    while d>tau
        initer=initer+1;
        if initer>3
            fprintf(fid,'Nummerical problems! Unexpected: more centering iterations than proved.\n');
            fprintf(fid,'primal object value=%e\n',sc_ip(c,x,K));
            fprintf(fid,'dual object value=%e\n',b'*y);
            return;
        end
        % centering step
        [Dx,Dy,Ds]=NTc(At,K,mu,x,s);
        x=x+Dx;
        y=y+Dy;
        s=s+Ds;
        d=delta(x,s,mu,K);
    end
    s=symmetry(s,K);
    rb=b-Ax(At',x,K);
    rc=c-At*y-s;
    dualitygap=sc_ip(x,s,K);
    normrb=norm(rb);
    normrc=sqrt(sc_ip(rc,rc,K));
    fprintf(fid,'  %d    %f   %e   %e   %e   %e   %e\n',initer,d,nu,mu,dualitygap,normrb,normrc);
end
fprintf(fid,'Problem solved successfully.\n');
fprintf(fid,'primal object value=%e\n',sc_ip(c,x,K));
fprintf(fid,'dual object value=%e\n',b'*y);
end


function [At,b,K,ord]=order(At,b,K)
% reorder columns of At, and b accordingly
% for handling spasity more efficiently

m=size(At,2);
nz=zeros(m,1);
for i=1:1:m
    nz(i)=nnz(At(K.lq+1:end,i));
end
[nzs,ord]=sort(nz,1,'descend');
At=At(:,ord);
b=b(ord);
snzs=nzs;
for i=m-1:-1:1
    snzs(i)=snzs(i)+snzs(i+1);
end
kappa=1.5;
n=sum(K.s);
n3=0;
for i=1:1:length(K.s)
    n3=n3+(K.s(i))^3;
end
K.s1=0;
for i=1:1:m
    d1=kappa*(n*nzs(i)+snzs(i))+n3;
    d2=kappa*(n*nzs(i)+(n+1)*snzs(i));
    d3=kappa*(2*kappa*nzs(i)+1)*snzs(i);
    if d1<=d2 && d1<=d3
        K.s1=i;
    else
        break;
    end
end
K.s2=K.s1;
for i=max([1,K.s1]):1:m
    d2=kappa*(n*nzs(i)+(n+1)*snzs(i));
    d3=kappa*(2*kappa*nzs(i)+1)*snzs(i);
    if d2<=d3
        K.s2=i;
    else
        break;
    end
end
end


function [zeta]=estimatezeta(At,b,c,K)
% estimate zeta
% Reference:
% On the implementation and usage of SDPT3
% --a Matlab software package for semidefinite-quadratic-linear programming, version 4.0

m=length(b);
zeta=10;
if K.l>0
    srl=sqrt(K.l);
    zeta=max([zeta,srl,norm(c(1:K.l))]);
    for j=1:1:m
        tmp=norm(At(1:K.l,j));
        zeta=max([zeta,tmp,srl*(1+abs(b(j)))/(1+tmp)]);
    end
end
index=K.l;
if isfield(K,'q')
    for i=1:1:length(K.q)
        srqi=sqrt(K.q(i));
        zeta=max([zeta,srqi,norm(c(index+1:index+K.q(i)))]);
        for j=1:1:m
            tmp=norm(At(index+1:index+K.q(i),j));
            zeta=max([zeta,tmp,srqi*(1+abs(b(j)))/(1+tmp)]);
        end
        index=index+K.q(i);
    end
end
if isfield(K,'s')
    for i=1:1:length(K.s)
        si2=K.s(i)*K.s(i);
        zeta=max([zeta,sqrt(K.s(i)),norm(c(index+1:index+si2))]);
        for j=1:1:m
            tmp=norm(At(index+1:index+si2,j));
            zeta=max([zeta,tmp,K.s(i)*(1+abs(b(j)))/(1+tmp)]);
        end
        index=index+si2;
    end
end
end


function [epsilon]=estimateepsilon(zeta,r)
% estimate epsilon
% we reduce the original duality gap by around 1e-16

index=ceil(log10(r*zeta*zeta*(1e-16)));
epsilon=10^index;
end


function [e]=identity(n,K)
% generate identity element

e=zeros(n,1);
if K.l>0
    e(1:K.l)=ones(K.l,1);
end
index=K.l;
if isfield(K,'q')
    for i=1:1:length(K.q)
        e(index+1)=1;
        index=index+K.q(i);
    end
end
if isfield(K,'s')
    for i=1:1:length(K.s)
        tmp=K.s(i)*K.s(i);
        e(index+1:index+tmp)=vec(eye(K.s(i)));
        index=index+tmp;
    end
end
end


function [Dxa,Dya,Dsa,Dxc,Dyc,Dsc,theta]=NTf(At,K,mu,rb,rc,x,s)
% solve the NT system for feasibility step

[g,w,v,vi]=sc_gwddi(x,s,K);
wAwAt=shur(At,w,K);
% right hand side for affine step
srmu=sqrt(mu);
v=v/srmu;
vi=vi*srmu;
br=-vi;
br_prc=br-sc_s(g,rc,K)/srmu;
rhs=(Ax(At',sc_s(g,br_prc,K,'T'),K)-rb/srmu);
% right hand side for centering step
br=vi-v;
rhs=[rhs, Ax(At',sc_s(g,br,K,'T'),K)];
% solve the linear system
xi=linearsolver(wAwAt,rhs);
% retrieve affine step
Dya=-xi(:,1)*srmu;
Dsa=rc-At*Dya;
dsa=sc_s(g,Dsa,K)/srmu;
dxa=br_prc-sc_s(g,At*xi(:,1),K);
% dxa=br-dsa;
Dxa=srmu*sc_s(g,dxa,K,'T');
% retrieve centering step
Dyc=-xi(:,2)*srmu;
Dsc=-At*Dyc;
dsc=sc_s(g,Dsc,K)/srmu;
% dxc=br-sc_s(g,At*xi,K);
dxc=br-dsc;
Dxc=srmu*sc_s(g,dxc,K,'T');
% estimate the update parameter
dxsaa=sc_jp(dxa,dsa,K);
dxscc=sc_jp(dxc,dsc,K);
dxsac=sc_jp(dxa,dsc,K)+sc_jp(dxc,dsa,K);
con=4-2*sqrt(3);
coe=[];
coe=[coe,sc_ip(dxsaa,dxsaa,K)];
coe=[coe,2*sc_ip(dxsaa,dxsac,K)];
coe=[coe,2*sc_ip(dxsaa,dxscc,K)+sc_ip(dxsac,dxsac,K)-con];
coe=[coe,2*(sc_ip(dxsac,dxscc,K)+con)];
coe=[coe,sc_ip(dxscc,dxscc,K)-con];
roo=roots(coe);
theta=0;
for i=1:length(roo)
    if isreal(roo(i))&&roo(i)>0&&(theta==0||theta>roo(i))
        theta=roo(i);
    end
end
if theta>1/(4*K.r) && theta<0.999
    vdxc=v+dxc;
    vdsc=v+dsc;
    vdxf=vdxc+theta*dxa;
    vdsf=vdsc+theta*dsa;
    d=delta(vdxf,vdsf,1-theta,K);
    sqrt2o2=sqrt(2)/2;
    while d<sqrt2o2-0.06 && theta<0.999
        vartheta=(sqrt2o2-d)/sqrt2o2*(1-theta);
        theta=theta+vartheta;
        vdxf=vdxc+theta*dxa;
        vdsf=vdsc+theta*dsa;
        inK=isinK(vdxf,vdsf,K);
        d=delta(vdxf,vdsf,1-theta,K);
        while d>sqrt2o2 || inK==0
            vartheta=vartheta/2;
            theta=theta-vartheta;
            vdxf=vdxc+theta*dxa;
            vdsf=vdsc+theta*dsa;
            inK=isinK(vdxf,vdsf,K);
            d=delta(vdxf,vdsf,1-theta,K);
        end
    end
end
end


function [Dx,Dy,Ds]=NTc(At,K,mu,x,s)
% solve the NT system for centering step with spares coefficient matrix

[g,w,v,vi]=sc_gwddi(x,s,K);
wAwAt=shur(At,w,K);
srmu=sqrt(mu);
v=v/srmu;
vi=vi*srmu;
br=vi-v;
xi=linearsolver(wAwAt,Ax(At',sc_s(g,br,K,'T'),K));
Dy=-xi*srmu;
Ds=-At*Dy;
% dx=br-sc_s(g,At*xi,K);
dx=br-sc_s(g,Ds,K)/srmu;
Dx=srmu*sc_s(g,dx,K,'T');
end


function [wAwAt]=shur(At,w,K)
% assemble the shur complement matrix

m=size(At,2);
if K.l>0
    Tmp=At(1:K.l,:);
    if issparse(At)
        wAwAt=Tmp'*sparse(1:K.l,1:K.l,w(1:K.l).*w(1:K.l))*Tmp;
    else
        wAwAt=Tmp'*diag(w(1:K.l).*w(1:K.l))*Tmp;
    end
else
    wAwAt=zeros(m);
end
if isfield(K,'q')
    wAwAt=wAwAt+2*shurq(At(K.l+1:K.lq,:),w(K.l+1:K.lq),K);
end
if isfield(K,'s')
    wAwAt=wAwAt+shurs(At(K.lq+1:end,:),w(K.lq+1:end),K);
end
end


function [xi]=linearsolver(wAwAt,rhs)
% solve the linear system

% perturb the shur complement matrix
if ~issparse(wAwAt)
    connum=rcond(wAwAt);
    if connum<1e-15
        d=max(1e-15,diag(wAwAt));
        %     para=min(1e-7, min(d)/max(d)*sqrt(1e-14/connum));
        %     para=10^min(round(log10(min(d)))-7, -4);
        %     if connum<1e-18
        %         para=para/10;
        %     end
        %     para=10^(min(log10(min(d))-7+(log10(connum)+15)/4, -4));
        para=10^(min(log10(min(d))-7, -4)+(log10(connum)+15)/4);
        wAwAt=wAwAt+para*eye(length(d));
    end
end
xi=wAwAt\rhs;
% solve the linear system by chol factorization
% R=chol(wAwAt);
% xi=R\(R'\rhs);
end


function [d]=delta(x,s,mu,K)
% delta

xi=sc_i(x,K);
si=sc_i(s,K);
d=sqrt(sc_ip(x,s,K)/mu+sc_ip(xi,si,K)*mu-2*K.r)/2;
end


function [x]=symmetry(x,K)
% symmetrize semidefinite matrices

if isfield(K,'s')
    index=K.lq;
    for i=1:1:length(K.s)
        tmp=K.s(i)*K.s(i);
        Tmp=mat(x(index+1:index+tmp));
        Tmp=(Tmp+Tmp')/2;
        x(index+1:index+tmp)=vec(Tmp);
        index=index+tmp;
    end
end
end


function [b]=Ax(A,x,K)
% Ax

if isfield(K,'q')
    x(K.l+1:K.lq)=2*x(K.l+1:K.lq);
end
b=A*x;
end


%% element functions for symmetric cone
function [f]=isinK(x,s,K)
% test if x and s in int K

f=1;
if K.l>0
    if min(x(1:K.l))<0 || min(s(1:K.l))<0
        f=0;
        return;
    end
end
index=K.l;
if isfield(K,'q')
    for i=1:1:length(K.q)
        tmp=K.q(i);
        if x(index+1)<norm(x(index+2:index+tmp)) || s(index+1)<norm(s(index+2:index+tmp))
            f=0;
            return;
        end
        index=index+tmp;
    end
end
if isfield(K,'s')
    for i=1:1:length(K.s)
        tmp=K.s(i)*K.s(i);
%         if min(eig(mat(x(index+1:index+tmp))))<0 || min(eig(mat(s(index+1:index+tmp))))<0
        if eigs(mat(x(index+1:index+tmp)),1,'sa')<0 || eigs(mat(s(index+1:index+tmp)),1,'sa')<0
            f=0;
            return;
        end
        index=index+tmp;
    end
end
end


function [ip]=sc_ip(x,s,K)
% inner product of symmetric cone

if isfield(K,'q')
    x(K.l+1:K.lq)=2*x(K.l+1:K.lq);
end
ip=x'*s;
end


function [r]=sc_rk(K)
% rank of symmetric cone

r=K.l;
if isfield(K,'q')
    r=r+2*length(K.q);
end
if isfield(K,'s')
    r=r+sum(K.s);
end
end


function [g,w,d,di]=sc_gwddi(x,s,K)
g=zeros(size(x));
w=g;
d=g;
di=g;
if K.l>0
    tmp=sqrt(x(1:K.l)./s(1:K.l));
    g(1:K.l)=sqrt(tmp);
    w(1:K.l)=tmp;
    tmp=sqrt(x(1:K.l).*s(1:K.l));
    d(1:K.l)=tmp;
    di(1:K.l)=1./tmp;
end
index=K.l;
if isfield(K,'q')
    for i=1:1:length(K.q)
        tmp=K.q(i);
        [tmp1,tmp2,tmp3,tmp4]=soc_gwddi(x(index+1:index+tmp),s(index+1:index+tmp));
        g(index+1:index+tmp)=tmp1;
        w(index+1:index+tmp)=tmp2;
        d(index+1:index+tmp)=tmp3;
        di(index+1:index+tmp)=tmp4;
        index=index+tmp;
    end
end
if isfield(K,'s')
    for i=1:1:length(K.s)
        tmp=K.s(i)*K.s(i);
        [tmp1,tmp2,tmp3,tmp4]=sd_gwddi(mat(x(index+1:index+tmp)),mat(s(index+1:index+tmp)));
        g(index+1:index+tmp)=vec(tmp1);
        w(index+1:index+tmp)=vec(tmp2);
        d(index+1:index+tmp)=vec(tmp3);
        di(index+1:index+tmp)=vec(tmp4);
        index=index+tmp;
    end
end
end


function [x_i]=sc_i(x,K)
% inverse of the interior of symmetric cone

x_i=zeros(size(x));
if K.l>0
    x_i(1:K.l)=1./x(1:K.l);
end
index=K.l;
if isfield(K,'q')
    for i=1:1:length(K.q)
        tmp=K.q(i);
        x_i(index+1:index+tmp)=soc_i(x(index+1:index+tmp));
        index=index+tmp;
    end
end
if isfield(K,'s')
    for i=1:1:length(K.s)
        tmp=K.s(i)*K.s(i);
        x_i(index+1:index+tmp)=vec(sd_i(mat(x(index+1:index+tmp))));
        index=index+tmp;
    end
end
end


function [gtxg]=sc_s(g,s,K,trans)
% scaling of symmetric cone

gtxg=zeros(size(s));
if K.l>0
    gs=g(1:K.l);
    gtxg(1:K.l)=s(1:K.l).*gs.*gs;
end
index=K.l;
if isfield(K,'q')
    for i=1:1:length(K.q)
        tmp=K.q(i);
        gtxg(index+1:index+tmp)=soc_p(g(index+1:index+tmp),s(index+1:index+tmp));
        index=index+tmp;
    end
end
if isfield(K,'s')
    for i=1:1:length(K.s)
        tmp=K.s(i)*K.s(i);
        G=mat(g(index+1:index+tmp));
        if nargin<4
            Tmp=G'*mat(s(index+1:index+tmp))*G;
        else
            Tmp=G*mat(s(index+1:index+tmp))*G';
        end
        Tmp=(Tmp+Tmp')/2;
        gtxg(index+1:index+tmp)=vec(Tmp);
        index=index+tmp;
    end
end
end


function [xs]=sc_jp(x,s,K)
% Jordan product of symmetric cone

xs=zeros(size(x));
if K.l>0
    xs(1:K.l)=x(1:K.l).*s(1:K.l);
end
index=K.l;
if isfield(K,'q')
    for i=1:1:length(K.q)
        xs(index+1:index+K.q(i))=soc_jp(x(index+1:index+K.q(i)),s(index+1:index+K.q(i)));
        index=index+K.q(i);
    end
end
if isfield(K,'s')
    for i=1:1:length(K.s)
        tmp=K.s(i)*K.s(i);
        xs(index+1:index+tmp)=vec(sd_jp(mat(x(index+1:index+tmp)),mat(s(index+1:index+tmp))));
        index=index+tmp;
    end
end
end


%% element functions for second-order cone
function [g,w,d,di]=soc_gwddi(x,s)
% scaling point of second-order cone

tmp=soc_sr(x);
w=soc_p(tmp,soc_i(soc_sr(soc_p(tmp,s))));
g=soc_sr(w);
d=soc_p(g,s);
di=soc_i(d);
end


function [x_i]=soc_i(x)
% inverse of the interior of second-order cone

tmp=x(1)*x(1)-x(2:end)'*x(2:end);
x_i=[x(1);-x(2:end)]/tmp;
% calculate by eigenvalue decomposition
% tmp1=norm(x(2:end));
% if tmp1<1e-32
%     x_i=[1/x(1);zeros(length(x)-1,1)];
% else
%     tmp2=x(2:end)/tmp1;
%     x_i=[1;-tmp2]/(2*(x(1)-tmp1))+[1;tmp2]/(2*(x(1)+tmp1));
% end
end


function [x_sr]=soc_sr(x)
% squareroot of second-order cone

tmp1=norm(x(2:end));
if tmp1<realmin
    x_sr=[sqrt(x(1));zeros(length(x)-1,1)];
else
    tmp2=x(2:end)/tmp1;
    x_sr=sqrt(x(1)-tmp1)/2*[1;-tmp2]+sqrt(x(1)+tmp1)/2*[1;tmp2];
end
end


function [pxs]=soc_p(x,s)
% Quadratic operator of second-order cone

% pxs=2*soc_jp(soc_jp(x,s),x)-soc_jp(soc_jp(x,x),s);
bxbx=x(2:end)'*x(2:end);
bxbs=x(2:end)'*s(2:end);
x1s1=x(1)*s(1);
pxs=[x(1)*x1s1+2*x(1)*bxbs+s(1)*bxbx;(x(1)*x(1)-bxbx)*s(2:end)+2*(x1s1+bxbs)*x(2:end)];
end


function [xs]=soc_jp(x,s)
% Jordan product of second-order cone

xs=[x'*s;x(1)*s(2:end)+s(1)*x(2:end)];
end


%% element functions for semidefinite matrices
function [G,W,D,Di]=sd_gwddi(X,S)
% scaling point of semidefinite matrices

Xc=chol(X)';
Tmp=Xc'*S*Xc;
Tmp=(Tmp+Tmp')/2;
[V,D2]=eig(Tmp);
d=sqrt(diag(D2));
D=diag(d);
Di=diag(1./d);
d12=sqrt(d);
G=Xc*V*diag(1./d12);
W=G*G';
W=(W+W')/2;
end


function [X_i]=sd_i(X)
% inverse of positive definite matrices

X_i=inv(X);
%X_i=X^(-1);
% keep X_i symmetric
%X_i=(X_i+X_i')/2;
end


function [XS]=sd_jp(X,S)
% Jordan product of semidefinite matrices

Tmp=X*S;
XS=(Tmp+Tmp')/2;
end


%% element functions for matrix and vector
function x=vec(X)  
% Given an m x n matrix X, this produces the vector x of length m*n that contains the columns of the matrix X, 
% stacked below each other.

[m n]=size(X);
x=reshape(X,m*n,1);
end


function X = mat(x,n)
% X= MAT(x,n) or Y = MAT(x) (the 2nd argument is optional)
% Given a vector of length n^2, this produces the n x n matrix X such that x = vec(X).  
% In other words, x contains the columns of the matrix X, stacked below each other.

if nargin<2
    n=floor(sqrt(length(x)));
    if (n*n)~=length(x)
        error('Argument X has to be a square matrix')
    end
end
X=reshape(x,n,n);
end